-- Addon:   LockoutChecker
-- Author:  Spielstein@Curse
-- Date:    18.08.2018



------------------------------------------
--    Functions                         --
------------------------------------------


--
function LockoutChecker:CollectOwnData()
	-- Variables --

	-- For-loop integer
	local i, j, dataArray
	-- Bucket variable
	local _

	-- Bariables for the GetSavedInstanceInfo(i) api-call
	local name, difficulty, locked, extended, numEncounters
	-- Bariables for the GetSavedInstanceEncounterInfo(i,j) API call
	local bossName, isKilled

	-- Iterating through all saved instance ids of a char
	for i = 1, GetNumSavedInstances() do

		-- API call
		name, _, _, difficulty, locked, extended, _, _, _, _, numEncounters, _ = GetSavedInstanceInfo(i)

		-- If dungeon is a mythic dungeon, a legion dungeon and as a lock or is extended
		if difficulty == 23 and LockoutChecker.Dungeons[name] and (locked or extended) then

			data = ""

			-- Iterate through all encounter in the dungeon
			for j = 1, numEncounters do

				-- API call
				bossName, _, isKilled, _ = GetSavedInstanceEncounterInfo(i, j)
				-- If the encounter is killed, give the encounter name a red color, green otherwise
				data = data .. (isKilled and isKilled and "|cffff0000" or "|cff008000") .. "x"
			end
			
			LockoutChecker.DATA.OwnData[LockoutChecker.Dungeons[name]+1] = data:format()
		end
	end
	LockoutChecker.DATA.OwnData[1] = LockoutChecker.OWN_NAME

	LockoutChecker.DATA.taint = false
end


--
function LockoutChecker:DecodeMessage(message)

	local _
	local match, key, value
	local data = {}

	for match in string.gmatch(message, "[^:%s]+") do
		_, _, key, value = string.find(match, "(.*);(.*)")
		data[tonumber(key)] = value
	end

	return data
end


--
function LockoutChecker:EncodeMessage(data)

	local k, v
	local message = ""

	for k, v in pairs(data) do
		message = message .. ":" .. k .. ";" .. v
	end
	
	return message
end


--
function LockoutChecker:ParseKeystoneString(keystoneString)

	local parsedInfo, subString, result
	local pattern =  "|Hkeystone:([0-9:]+)|h"

	if keystoneString then
		subString = string.match(keystoneString, pattern)
		if subString then
			parsedInfo = { strsplit(":", subString) }

			result = {}
			result["mapID"] = tonumber(parsedInfo[1])
			result["mythicLevel"] = tonumber(parsedInfo[2])
			result["active"] = (parsedInfo[3] == "1")
			result["affix"] = {}
			result["affix"][1] = parsedInfo[4]
			if parsedInfo[5] then result["affix"][2] = parsedInfo[5] end
			if parsedInfo[6] then result["affix"][3] = parsedInfo[6] end
			print(C_ChallengeMode.GetMapInfo(result["mapID"]))

			return result
		end
	end
	return nil
end


--
function LockoutChecker:SendDataTo(receiver, channel)
	local message = LockoutChecker:EncodeMessage(LockoutChecker.DATA.OwnData)
	C_ChatInfo.SendAddonMessage("LockoutChecker", message, channel)
end


--
function LockoutChecker:ParseIncomingData(message, sender, channel)
	local data = LockoutChecker:DecodeMessage(message)
	if channel == "PARTY" then
		data[1] = LockoutChecker.FRIENDS_ICON:format() .. data[1]
	end

	LockoutChecker.DATA[channel][sender] = data
end


--
function LockoutChecker:SendRequestLockout()
	
	-- clean up
	LockoutChecker.DATA.OwnData = {}
	LockoutChecker.DATA.PARTY = {}
	LockoutChecker.DATA.GUILD = {}

	-- get own data
	LockoutChecker:CollectOwnData()
	LockoutChecker:UpdateRows()

	-- send requests
	C_ChatInfo.SendAddonMessage("LockoutChecker", "RequestLockout", "PARTY")
	C_ChatInfo.SendAddonMessage("LockoutChecker", "RequestLockout", "GUILD")
end


--
function LockoutChecker:ReceiveMessage(event, prefix, message, channel, sender)
	if event == "CHAT_MSG_ADDON" and prefix == "LockoutChecker" and sender ~= LockoutChecker.OWN_NAME_AND_REALM then
		if message == "RequestLockout" then
			--if LockoutChecker.DATA.taint then
			LockoutChecker:CollectOwnData()
			--end
			LockoutChecker:SendDataTo(sender, channel)
		else
			LockoutChecker:ParseIncomingData(message, sender, channel)
			LockoutChecker:UpdateRows()
		end
	end
end


--
function LockoutChecker:KeystoneSyncButtonPressed()

	LockoutChecker.gui.keystoneSyncButton:Disable()
	LockoutChecker.gui.lockoutSyncButton:Disable()
	LockoutChecker.gui.keystoneSyncButton:SetScript("OnUpdate", function (self, elapsed)

			self.elapsedTime = self.elapsedTime + elapsed

		    if self.elapsedTime >= self.maxTime then

		    	self:SetText(self.stdText)
		        self:Enable()
		        self.elapsedTime = 0
		        self:SetScript("OnUpdate", nil)

		        self:GetParent().lockoutSyncButton:Enable()

		    else
		    	self:SetText(string.format("%.0f", self.maxTime - self.elapsedTime))
		    end
		end)

	-- TODO
end

--
function LockoutChecker:LockoutSyncButtonPressed()

	LockoutChecker.gui.lockoutSyncButton:Disable()
	LockoutChecker.gui.keystoneSyncButton:Disable()
	LockoutChecker.gui.lockoutSyncButton:SetScript("OnUpdate", function (self, elapsed)

			self.elapsedTime = self.elapsedTime + elapsed

		    if self.elapsedTime >= self.maxTime then

		    	self:SetText(self.stdText)
		        self:Enable()
		        self.elapsedTime = 0
		        self:SetScript("OnUpdate", nil)

		        self:GetParent().keystoneSyncButton:Enable()

		    else
		    	self:SetText(string.format("%.0f", self.maxTime - self.elapsedTime))
		    end
		end)

	LockoutChecker:SendRequestLockout()
end


--
function LockoutChecker:HookScript(button, down)
	if button == "RightButton" then
		if LockoutChecker.gui:IsShown() then
	    	LockoutChecker.gui:Hide()
	    else
	    	LockoutChecker.gui:Show()
	    end
	elseif button == "LeftButton" then
		LockoutChecker:OldHookedScript(button, down)
	end
end


--
LockoutChecker.ScrollScript = function (self, scrollValue)
	
	local value
	local _

	local collectedData = {}
	table.insert(collectedData, LockoutChecker.DATA.OwnData)

	for _, value in pairs(LockoutChecker.DATA.PARTY) do
        table.insert(collectedData, value)
	end

	for key, value in pairs(LockoutChecker.DATA.GUILD) do
        if not LockoutChecker.DATA.PARTY[key] then
        	table.insert(collectedData, value)
        end
	end

	local flooredValue = math.floor(scrollValue)
	local i
	for i = 1, 20 do
		LockoutChecker.gui.rows[i]:SetText(collectedData[i+flooredValue])
	end
end


--
function LockoutChecker:UpdateRows()
	
	local i,j
	local value
	local _

	-- array for collecting all data
	local collectedData = {}
	
	-- add personal data if existent
	local next = next
	if next(LockoutChecker.DATA.OwnData) then
	  	table.insert(collectedData, LockoutChecker.DATA.OwnData)
	end

	-- add party data
	for _, value in pairs(LockoutChecker.DATA.PARTY) do
	    table.insert(collectedData, value)
	end

	-- add guild data
	for key, value in pairs(LockoutChecker.DATA.GUILD) do
		if not LockoutChecker.DATA.PARTY[key] then
        	table.insert(collectedData, value)
        end
	end

	-- iterate through collected data
	local numData = #collectedData
	if numData < 20 then
		for i = 1, numData do
			LockoutChecker.gui.rows[i]:Show()
			LockoutChecker.gui.rows[i]:SetText(collectedData[i])
		end
		for j = numData+1, 20 do
			LockoutChecker.gui.rows[j]:Hide()
		end
		LockoutChecker.gui.scrollbar:Hide()
	else
		local flooredValue = math.floor(LockoutChecker.gui.scrollbar:GetValue())
		local i
		for i = 1, 20 do
			LockoutChecker.gui.rows[i]:SetText(collectedData[i+flooredValue])
			LockoutChecker.gui.rows[i]:Show()
		end
		LockoutChecker.gui.scrollbar:Show()
		LockoutChecker.gui.scrollbar:SetMinMaxValues(0, math.max((#collectedData-#LockoutChecker.gui.rows), 0))
	end
	if numData == 0 then
		LockoutChecker.gui.dungeonRow:Hide()
	else
		LockoutChecker.gui.dungeonRow:Show()
	end
end

LockoutChecker.gui = LockoutChecker:CreateGUI(UIParent, LockoutChecker.LockoutSyncButtonPressed, LockoutChecker.KeystoneSyncButtonPressed, LockoutChecker.ScrollScript)

LockoutChecker.DATA = {}
LockoutChecker.DATA.OwnData = {}
LockoutChecker.DATA.PARTY = {}
LockoutChecker.DATA.GUILD = {}
LockoutChecker.DATA.taint = true

LockoutChecker:UpdateRows()

LockoutChecker.gui.scrollbar:SetMinMaxValues(0, 0)


-- Hooking showing/hiding the gui on the rightclick on the blizz-ui-friendsmicrobutton
QuickJoinToastButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
LockoutChecker.OldHookedScript = QuickJoinToastButton:GetScript("OnClick")
QuickJoinToastButton:SetScript("OnClick", LockoutChecker.HookScript)

--
LockoutChecker.mainFrame:RegisterEvent("CHAT_MSG_ADDON")
LockoutChecker.mainFrame:SetScript("OnEvent", LockoutChecker.ReceiveMessage)
C_ChatInfo.RegisterAddonMessagePrefix("LockoutChecker")

LockoutChecker.gui:RegisterEvent("ENCOUNTER_END")

-- EVENT           arg1         arg2           arg3          arg4       arg5
-- ENCOUNTER_END   encounterID  encounterName  difficultyID  groupSize  success
LockoutChecker.gui:SetScript("OnEvent", function(self, event, encounterID, encounterName, difficultyID, groupSize, success) 
	if event == "ENCOUNTER_END" and success == 1 and difficultyID == 23 then
		LockoutChecker.DATA.taint = true
		print("end and  success")
	end
end)


------------------------------------------
--		SlashCommand					--
------------------------------------------

SLASH_LOCKOUTCHECKER1 = "/lc"
SLASH_LOCKOUTCHECKER2 = "/lockoutchecker"
SlashCmdList["LOCKOUTCHECKER"] = function(msg)
    if LockoutChecker.gui:IsShown() then
    	LockoutChecker.gui:Hide()
    else
    	LockoutChecker.gui:Show()
    end
end
------------------------------------------
--		End of Script					--
------------------------------------------
