-- Addon:	LockoutChecker
-- Author:	Spielstein@Curse
-- Date:    18.08.2018



------------------------------------------
--		GUI								--
------------------------------------------

-- constructor for the gui
function LockoutChecker:CreateGUI(parentFrame, lockoutSyncButtonScript, keystoneSyncButtonScript, scrollScript)


	-- MainFrame
	local gui = CreateFrame("Frame", "LockoutCheckerGUI", parentFrame, "TooltipBorderedFrameTemplate")
	gui:SetSize(1100, 600)
	gui:SetPoint("CENTER")
	gui:SetFrameStrata("HIGH")

	-- exit button
	local closeButton = CreateFrame("Button", nil, gui, "UIPanelCloseButton")
	closeButton:SetPoint("TOPRIGHT")
	closeButton:SetScript("OnClick", function (self)
		self:GetParent():Hide()
	end)
	gui.closeButton = closeButton

	-- lockout sync button
	local lockoutSyncButton = CreateFrame("Button", nil, gui, "UIPanelButtonTemplate")
	lockoutSyncButton.stdText = "Lockouts"
	lockoutSyncButton:SetText(lockoutSyncButton.stdText)
	lockoutSyncButton:SetPoint("TOPLEFT", gui, "TOPLEFT", 10, -10)
	lockoutSyncButton:SetWidth(100)
	lockoutSyncButton:SetHeight(25)
	lockoutSyncButton:SetScript("OnClick", lockoutSyncButtonScript)

	lockoutSyncButton.elapsedTime = 0
	lockoutSyncButton.maxTime = 3

	gui.lockoutSyncButton = lockoutSyncButton


	-- keystone sync button
	local keystoneSyncButton = CreateFrame("Button", nil, gui, "UIPanelButtonTemplate")
	keystoneSyncButton.stdText = "KeyStones"
	keystoneSyncButton:SetText(keystoneSyncButton.stdText)
	keystoneSyncButton:SetPoint("LEFT", gui.lockoutSyncButton, "RIGHT", 10, 0)
	keystoneSyncButton:SetWidth(100)
	keystoneSyncButton:SetHeight(25)
	keystoneSyncButton:SetScript("OnClick", keystoneSyncButtonScript)

	keystoneSyncButton.elapsedTime = 0
	keystoneSyncButton.maxTime = 3

	gui.keystoneSyncButton = keystoneSyncButton
	keystoneSyncButton:Hide() -- TODO


	-- Scrollframe
	local scrollframe = CreateFrame("ScrollFrame", nil, gui)
	scrollframe:SetPoint("TOPLEFT", 10, -35)
	scrollframe:SetPoint("BOTTOMRIGHT", -10, 10)
	gui.scrollframe = scrollframe


	-- Scrollbar
	local scrollbar = CreateFrame("Slider", nil, scrollframe, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("TOPLEFT", gui, "TOPRIGHT", 4, -16)
	scrollbar:SetPoint("BOTTOMLEFT", gui, "BOTTOMRIGHT", 0, 16)
	scrollbar:SetMinMaxValues(0, 800)
	scrollbar:SetValueStep(1)
	scrollbar.scrollStep = 1
	scrollbar:SetValue(0)
	scrollbar:SetWidth(16)

	-- Functionality for the scrollbar
	scrollbar:SetScript("OnValueChanged", scrollScript)

	-- Scroll wheel functionality
	scrollframe:EnableMouseWheel(true)
  	scrollframe:SetScript("OnMouseWheel",
      function (self, value)
          local minV, maxV = scrollbar:GetMinMaxValues()
          local currV = scrollbar:GetValue()

          if value > 0 then -- and currV - minV > 0 then
				scrollbar:SetValue(currV - 1)
          end
          if value < 0 then -- and maxV - currV > 0 then
				scrollbar:SetValue(currV + 1)
          end
      end)

    -- ScrollBarBG
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND")
	scrollbg:SetAllPoints(scrollbar)
	scrollbg:SetColorTexture(0, 0, 0, 0.4)
	gui.scrollbar = scrollbar

	-- Content frame
	local content = CreateFrame("Frame", nil, scrollframe)
	content:SetSize(128, 128)
	scrollframe.content = content

	scrollframe:SetScrollChild(content)


	-- Hide GUI on StartUp
	gui:Hide()
	-- Make GUI movable
	gui:RegisterForDrag("LeftButton")
	gui:SetMovable(true)
	gui:EnableMouse(true)
	gui:SetScript("OnDragStart", gui.StartMoving)
	gui:SetScript("OnDragStop", gui.StopMovingOrSizing)
	-- Make GUI closable by Escape key
	tinsert(UISpecialFrames, gui:GetName())

	gui.rows = {}
	gui.dungeonRow = CreateFrame("Frame", nil, gui.scrollframe, nil)

	local row, anchor
	local columnWidth, rowHeight, numColumns
	rowHeight = 25
	numColumns = 11

	local x
	for x = 1, 20 do
		-- create row
		local i
		

		row = CreateFrame("Frame", nil, gui.scrollframe.content, nil)
		row:SetSize(gui.scrollframe:GetWidth()-10, rowHeight)
		columnWidth = (row:GetWidth()-100)/(numColumns-1)

		if #gui.rows > 0 then
			row:SetPoint("TOP", gui.rows[#gui.rows], "BOTTOM", 0, -1)
		else
			row:SetPoint("TOPLEFT", gui.scrollframe.content, "TOPLEFT", 5, -30)
		end

		anchor = row
		row.columns = {}

		for i = 0, numColumns-1 do
			row.columns[i] = CreateFrame("Frame", nil, row, nil)
			row.columns[i]:SetSize((i==0 and 99 or columnWidth-1), rowHeight)
			row.columns[i]:SetPoint("LEFT", anchor, (i==0 and "LEFT" or "RIGHT"), 1, 0)

			row.columns[i].background = row.columns[i]:CreateTexture(nil, "BACKGROUND")
			row.columns[i].background:SetAllPoints()
			if i % 2 == 0 then
				row.columns[i].background:SetColorTexture(0.388, 0.722, 1, 0.5)
			else
				row.columns[i].background:SetColorTexture(0.306, 0.58, 0.804, 0.5)
			end

			row.columns[i].text = row.columns[i]:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
			row.columns[i].text:SetPoint("CENTER", row.columns[i], "CENTER")

			anchor = row.columns[i]
		end

		row.SetText = function(self, data)
			local i
			for i = 0, #self.columns do
				self.columns[i].text:SetText((data[i+1] ~= nil and data[i+1] or string.format("|cff008000x")))
				if i % 2 == 0 then
					row.columns[i].background:SetColorTexture(0.388, 0.722, 1, 0.5)
				else
					row.columns[i].background:SetColorTexture(0.306, 0.58, 0.804, 0.5)
				end
				--if i % 2 == 0 then
				--	row.columns[i].background:SetColorTexture(0.251, 1, 0,251, 0.5)
				--else
				--	row.columns[i].background:SetColorTexture(0.459, 1, 0.459, 0.5)
				--end
			end
		end

		row.SetLockoutMode = function(self)
			for i = 0, #self.columns do
				self.columns[i]:SetSize((i==0 and 99 or columnWidth-1), rowHeight)
				self.columns[i]:Show()
			end
		end

		row.SetMPlusMode = function(self)
			self.columns[0]:SetSize(99, rowHeight)
			self.columns[1]:SetSize(45, rowHeight)
			self.columns[2]:SetSize(self:GetWidth()-150-3*(columnWidth-1), rowHeight)
			self.columns[3]:SetSize(columnWidth-1, rowHeight)
			self.columns[4]:SetSize(columnWidth-1, rowHeight)
			self.columns[5]:SetSize(columnWidth-1, rowHeight)
			for i = 6, #self.columns do
				self.columns[i]:Hide()
			end
		end

		table.insert(gui.rows, row)
	end

	-- dungeon row
	gui.dungeonRow:SetSize(gui.scrollframe:GetWidth()-10, rowHeight)
	columnWidth = (gui.dungeonRow:GetWidth()-100)/(numColumns-1)

	gui.dungeonRow:SetPoint("TOPLEFT", gui.scrollframe.content, "TOPLEFT", 5, -4)
	anchor = gui.dungeonRow

	gui.dungeonRow.columns = {}
	local dungeonRowTextData = {
		["enUS"] = {"Atal'Dazor", "Freehold", "Kings' Rest", "Shrine of the\nStorms", "Siege of\nBoralus", "Sethraliss", "MOTHERLODE", "Underrot", "Tol Dagor", "Waycrest\nManor"},
		["enGB"] = {"Atal'Dazor", "Freehold", "Kings' Rest", "Shrine of the\nStorms", "Siege of\nBoralus", "Sethraliss", "MOTHERLODE", "Underrot", "Tol Dagor", "Waycrest\nManor"},
		["deDE"] = {"Atal'Dazor", "Freihafen", "Die Königsruh", "Der Schrein\ndes Sturms", "Die Belagerung\nvon Boralus", "Der Tempel\nvon Sethraliss", "Das\nRIESENFLÖZ!!", "Der\nTiefenpfuhl", "Tol\nDagor", "Das\nKronsteiganwesen"}
	}

	for i = 0, numColumns-1 do
		gui.dungeonRow.columns[i] = CreateFrame("Frame", nil, gui.dungeonRow, nil)
		gui.dungeonRow.columns[i]:SetSize((i==0 and 99 or columnWidth-1), rowHeight)
		gui.dungeonRow.columns[i]:SetPoint("LEFT", anchor, (i==0 and "LEFT" or "RIGHT"), 1, 0)

		gui.dungeonRow.columns[i].background = gui.dungeonRow.columns[i]:CreateTexture(nil, "BACKGROUND")
		gui.dungeonRow.columns[i].background:SetAllPoints()
		if i % 2 == 0 then
			gui.dungeonRow.columns[i].background:SetColorTexture(0.388, 0.722, 1, 0.8)
		else
			gui.dungeonRow.columns[i].background:SetColorTexture(0.306, 0.58, 0.804, 0.8)
		end

		gui.dungeonRow.columns[i].text = gui.dungeonRow.columns[i]:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
		gui.dungeonRow.columns[i].text:SetPoint("CENTER", gui.dungeonRow.columns[i], "CENTER")
		gui.dungeonRow.columns[i].text:SetText(dungeonRowTextData[LockoutChecker.LOCALE][i])

		anchor = gui.dungeonRow.columns[i]
	end
	gui.dungeonRow.columns[0]:Hide()

	-- Return finished GUI
	return gui

end



------------------------------------------
--		End of Script					--
------------------------------------------
